package be.iminds.ilabt.jfed.experimenter_gui.gts_editor.toolbox;

import be.iminds.ilabt.jfed.experimenter_gui.editor.ToolboxItem;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.views.EditableGTSCanvas;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.geometry.Point2D;
import javafx.scene.control.Label;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.DataFormat;

import javax.annotation.Nonnull;
import java.io.IOException;
import java.net.URL;

/**
 * User: twalcari
 * Date: 10/18/13
 * Time: 5:16 PM
 */
public abstract class GTSToolboxItem extends Label  implements Comparable<GTSToolboxItem> {

    @FXML
    protected ImageView imageView;


    private final Image dragImage;
    private static DataFormat GTS_DATA_FORMAT = new DataFormat("jfed/gts-toolbox-item");

    public GTSToolboxItem(String id, String text, Image image, Image dragImage) {
        setId(id);
        URL location = ToolboxItem.class.getResource("ToolboxItem.fxml");
        assert (location != null);
        FXMLLoader fxmlLoader = new FXMLLoader(location, null);
        fxmlLoader.setRoot(this);
        fxmlLoader.setController(this);

        try {
            fxmlLoader.load();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        this.setText(text);
        imageView.setImage(image);
        this.dragImage = dragImage;
    }

    public GTSToolboxItem(String newId, GTSToolboxItem item) {
        this(newId, item.getText(), item.getImageView().getImage(), item.getDragImage());
    }

    public static DataFormat getDragDataFormat() {
        return GTS_DATA_FORMAT;
    }

    public ImageView getImageView() {
        return imageView;
    }

    public Image getDragImage() {
        return dragImage;
    }

    @Override
    public String toString() {
        return "ToolboxItem{" +
                "imageView=" + imageView +
                '}';
    }

    public abstract void onAddedToCanvas(EditableGTSCanvas canvas, Point2D location);

    public abstract int getWeight();

    @Override
    public int compareTo(@Nonnull GTSToolboxItem o) {
        return getWeight() - o.getWeight();
    }

}
